@@ -2,25 +2,27 @@ use libloading::*;
 use const_cstr::*;
 use once_cell::sync::Lazy;
 use crate::vulkan::*;
+use crate::error::MiraError;
+use crate::error::MiraError::{CommandLoadError, BackendError};
 use std::os::raw::c_char;
 
 #[cfg(target_os = "linux")]
-const LIB_NAME:&str = "libvulkan.so.1";
+pub const LIB_NAME:&str = "libvulkan.so.1";
 
 #[cfg(target_os = "android")]
-const LIB_NAME:&str = "libvulkan.so";
+pub const LIB_NAME:&str = "libvulkan.so";
 
 #[cfg(target_os = "macos")]
-const LIB_NAME:&str = "libvulkan.1.dylib";
+pub const LIB_NAME:&str = "libvulkan.1.dylib";
 
 #[cfg(target_os = "ios")]
-const LIB_NAME:&str = "libvulkan.1.dylib";
+pub const LIB_NAME:&str = "libvulkan.1.dylib";
 
 #[cfg(target_os = "windows")]
-const LIB_NAME:&str = "vulkan-1.dll";
+pub const LIB_NAME:&str = "vulkan-1.dll";
 
 /// Command errors
-type CommandError = libloading::Error;
+//type CommandError = libloading::Error;
 
 /// Instance and device command loader.
 pub struct Command {
@@ -35,23 +37,23 @@ impl Command {
     ///
     /// From [libloading::Library::new].
     ///
-    pub unsafe fn new(lib_path:&str) -> Result<Self, CommandError> {
+    pub unsafe fn new(lib_path:&str) -> Result<Self, MiraError> {
         let lib = match Library::new(lib_path) {
-            Err(e) => return Err(e),
+            Err(e) => return Err(BackendError(e)),
             Ok(lib) => lib
         };
 
         Ok( Self {
-            library: lib
+            library: lib,
         })
     }
 
     /// Gets a pointer for vkGetInstanceProcAddr.
-    pub fn instance(&self) -> Result<Symbol<PFN_vkGetInstanceProcAddr>, CommandError> {
+    pub fn instance(&self) -> Result<Symbol<PFN_vkGetInstanceProcAddr>, MiraError> {
         let sym = unsafe {
             match self.library.get::<PFN_vkGetInstanceProcAddr>(b"vkGetInstanceProcAddr\0") {
                 Ok(sym) => sym,
-                Err(e) => return Err(e)
+                Err(e) => return Err(BackendError(e))
             }
         };
 
@@ -59,11 +61,11 @@ impl Command {
     }
 
     /// Gets a pointer for vkGetDeviceProcAddr.
-    pub fn device(&self) -> Result<Symbol<PFN_vkGetDeviceProcAddr>, CommandError> {
+    pub fn device(&self) -> Result<Symbol<PFN_vkGetDeviceProcAddr>, MiraError> {
         let sym = unsafe {
             match self.library.get::<PFN_vkGetDeviceProcAddr>(b"vkGetDeviceProcAddr\0") {
                 Ok(sym) => sym,
-                Err(e) => return Err(e)
+                Err(e) => return Err(BackendError(e))
             }
         };
 
@@ -84,12 +86,17 @@ static INTERNAL_LOADER:Lazy<Command> = Lazy::new(|| {
 /// # Safety
 /// If function pointer and T have different size a undefined behavior can occur.
 ///
-pub unsafe fn instance<T: Sized>(instance: VkInstance, command: ConstCStr) -> T {
+pub unsafe fn instance<T: Sized>(instance: VkInstance, command: ConstCStr) -> Result<T, MiraError> {
     static I:Lazy<Symbol<'static, PFN_vkGetInstanceProcAddr>> = Lazy::new(|| {
         INTERNAL_LOADER.instance().unwrap()
     });
 
-    std::mem::transmute_copy(&I(instance, command.as_ptr() as *const c_char))
+    let pfn = I(instance, command.as_ptr() as *const c_char) as *const PFN_vkVoidFunction;
+    if pfn == std::ptr::null() {
+        return Err(CommandLoadError { command: command.to_str() });
+    }
+
+    Ok(std::mem::transmute_copy(&pfn))
 }
 
 /// Gets from `device` a device command pointer for `command`
@@ -97,10 +104,15 @@ pub unsafe fn instance<T: Sized>(instance: VkInstance, command: ConstCStr) -> T
 /// # Safety
 /// Using a incorrect type may cause undefined behavior.
 ///
-pub unsafe fn device<T: Sized>(device: VkDevice, command: ConstCStr) -> T {
-    static I:Lazy<Symbol<'static, PFN_vkGetDeviceProcAddr>> = Lazy::new(|| {
+pub unsafe fn device<T: Sized>(device: VkDevice, command: ConstCStr) -> Result<T, MiraError> {
+    static D:Lazy<Symbol<'static, PFN_vkGetDeviceProcAddr>> = Lazy::new(|| {
         INTERNAL_LOADER.device().unwrap()
     });
 
-    std::mem::transmute_copy(&I(device, command.as_ptr() as *const c_char))
+    let pfn = D(device, command.as_ptr() as *const c_char) as *const PFN_vkVoidFunction;
+    if pfn == std::ptr::null() {
+        return Err(CommandLoadError { command: command.to_str() });
+    }
+
+    Ok(std::mem::transmute_copy(&pfn))
 }
